/*
 * opalvxml.h
 *
 * Header file for IVR code
 *
 * A H.323 IVR application.
 *
 * Copyright (C) 2002 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: opalvxml.h,v $
 * Revision 1.4  2002/07/03 01:53:58  craigs
 * Added newline to the end of the file for Linux
 *
 * Revision 1.3  2002/07/02 06:32:04  craigs
 * Added recording functions
 *
 * Revision 1.2  2002/06/28 02:42:54  craigs
 * Fixed problem with G.723.1 file codec not identified as native format
 *
 * Revision 1.1  2002/06/27 05:44:39  craigs
 * Initial version
 *
 * Revision 1.2  2002/06/26 09:05:28  csoutheren
 * Added ability to utter various "sayas" types within prompts
 *
 * Revision 1.1  2002/06/26 01:13:53  csoutheren
 * Disassociated VXML and Opal/OpenH323 specific elements
 *
 *
 */

#ifndef _OpenIVR_OPALVXML_H
#define _OpenIVR_OPALVXML_H

#include <ptclib/vxml.h>
#include <opalwavfile.h>
#include <ptclib/delaychan.h>
#include <h323caps.h>
#include <h245.h>
#include <h323con.h>

class G7231_File_Codec : public H323AudioCodec
{
  PCLASSINFO(G7231_File_Codec, H323AudioCodec);

  public:
    G7231_File_Codec(Direction dir);

    unsigned GetBandwidth() const;
    static int GetFrameLen(int val);
      
    BOOL Read(BYTE * buffer, unsigned & length, RTP_DataFrame &);
    BOOL Write(const BYTE * buffer, unsigned length, const RTP_DataFrame & rtp, unsigned & frames);

    BOOL IsRawDataChannelNative() const
      { return TRUE; }


  protected:
    PAdaptiveDelay incomingDelay;
};  

class G7231_File_Capability : public H323AudioCapability
{
  PCLASSINFO(G7231_File_Capability, H323AudioCapability)

  public:
    G7231_File_Capability();

    unsigned GetSubType() const { return H245_AudioCapability::e_g7231; }
    PString GetFormatName() const { return "G.723.1"; }

    H323Codec * CreateCodec(H323Codec::Direction direction) const;

    BOOL OnSendingPDU(H245_AudioCapability & cap, unsigned packetSize) const;
    BOOL OnReceivedPDU(const H245_AudioCapability & pdu, unsigned & packetSize);
    PObject * Clone() const;
};

//////////////////////////////////////////////////////////////////

class PVXMLChannel : public PIndirectChannel
{
  PCLASSINFO(PVXMLChannel, PIndirectChannel);
  public:
    PVXMLChannel(PVXMLSession & _vxml)
      : vxml(_vxml) { }

    virtual void DelayFrame(PINDEX msecs) = 0;

  protected:
    PVXMLSession & vxml;
    PMutex vxmlMutex;
    PAdaptiveDelay delay;
};

//////////////////////////////////////////////////////////////////

class PVXMLOutgoingChannel;

class PVXMLQueueItem : public PObject
{
  PCLASSINFO(PVXMLQueueItem, PObject);
  public:
    virtual void Play(PVXMLOutgoingChannel & outgoingChannel) = 0;
};

class PVXMLQueueDataItem : public PVXMLQueueItem
{
  PCLASSINFO(PVXMLQueueDataItem, PVXMLQueueItem);
  public:
    PVXMLQueueDataItem(const PBYTEArray & _data)
      : data(_data)
    { }

    void Play(PVXMLOutgoingChannel & outgoingChannel);

  protected:
    PBYTEArray data;
};

class PVXMLQueueFilenameItem : public PVXMLQueueItem
{
  PCLASSINFO(PVXMLQueueFilenameItem, PObject);
  public:
    PVXMLQueueFilenameItem(const PFilePath & _fn)
      : fn(_fn)
    { }

    void Play(PVXMLOutgoingChannel & outgoingChannel);

  protected:
    PFilePath fn;
};

PQUEUE(PVXMLQueue, PVXMLQueueItem);

class PVXMLOutgoingChannel : public PVXMLChannel
{
  PCLASSINFO(PVXMLOutgoingChannel, PVXMLChannel);

  public:
    PVXMLOutgoingChannel(PVXMLSession & vxml);

    // overrides from PIndirectChannel
    virtual BOOL Read(void * buffer, PINDEX amount);
    BOOL Close();

    // new functions
    virtual PWAVFile * CreateWAVFile(const PFilePath & fn)
      { return new PWAVFile(fn, PFile::ReadOnly); }

    virtual BOOL AdjustFrame(void * buffer, PINDEX amount);
    virtual void QueueFile(const PString & fn);
    virtual void QueueData(const PBYTEArray & data);
    virtual void PlayFile(PFile * chan);
    virtual void FlushQueue();
    virtual BOOL IsPlaying() const   { return (playQueue.GetSize() > 0) || playing ; }

  protected:
    // new functions
    virtual BOOL ReadFrame(PINDEX amount)= 0;
    virtual void CreateSilenceFrame(PINDEX amount) = 0;

    PVXMLQueue playQueue;
    BOOL playing;

    PBYTEArray frameBuffer;
    PINDEX frameLen, frameOffs;

    int silentCount;
    int totalData;
};

//////////////////////////////////////////////////////////////////

class PVXMLIncomingChannel : public PVXMLChannel
{
  PCLASSINFO(PVXMLIncomingChannel, PVXMLChannel)

  public:
    PVXMLIncomingChannel(PVXMLSession & vxml);
    ~PVXMLIncomingChannel();

    // overrides from PIndirectChannel
    BOOL Write(const void * buf, PINDEX len);

    // new functions
    BOOL StartRecording(const PFilePath & fn);
    BOOL EndRecording();
    BOOL IsRecording() const { return wavFile != NULL; }

    virtual PWAVFile * CreateWAVFile(const PFilePath & fn)
      { return new PWAVFile(fn, PFile::WriteOnly, PWAVFile::fmt_uLaw); }

  protected:
    // new functions
    virtual BOOL WriteFrame(const void * buf, PINDEX len) = 0;
    PWAVFile * wavFile;
};

//////////////////////////////////////////////////////////////////

class OpalVXMLSession;

class OpalVXMLOutgoingChannel : public PVXMLOutgoingChannel
{
  PCLASSINFO(OpalVXMLOutgoingChannel, PVXMLOutgoingChannel);

  public:
    OpalVXMLOutgoingChannel(OpalVXMLSession & vxml);

    // new functions
    PWAVFile * CreateWAVFile(const PFilePath & fn);

  protected:
    virtual unsigned GetWavFileType() const = 0;
    virtual BOOL IsWAVFileValid(OpalWAVFile & chan) = 0;
};

class OpalVXMLOutgoingChannelPCM : public OpalVXMLOutgoingChannel
{
  PCLASSINFO(OpalVXMLOutgoingChannelPCM, OpalVXMLOutgoingChannel);

  public:
    OpalVXMLOutgoingChannelPCM(OpalVXMLSession & vxml);

    unsigned GetWavFileType() const
     { return PWAVFile::PCM_WavFile; }

  protected:
    // overrides from PVXMLOutgoingChannel
    void DelayFrame(PINDEX len);
    BOOL IsWAVFileValid(OpalWAVFile & chan);
    BOOL ReadFrame(PINDEX amount);
    void CreateSilenceFrame(PINDEX amount);
};

class OpalVXMLOutgoingChannelG7231 : public OpalVXMLOutgoingChannel
{
  PCLASSINFO(OpalVXMLOutgoingChannelG7231, OpalVXMLOutgoingChannel);
  public:
    OpalVXMLOutgoingChannelG7231(OpalVXMLSession & vxml);
    void QueueFile(const PString & ofn);

  protected:
    unsigned GetWavFileType() const
      { return PWAVFile::fmt_VivoG7231; }

    // overrides from PVXMLOutgoingChannel
    void DelayFrame(PINDEX len);
    BOOL IsWAVFileValid(OpalWAVFile & chan);
    BOOL ReadFrame(PINDEX amount);
    void CreateSilenceFrame(PINDEX amount);
};

//////////////////////////////////////////////////////////////////

class OpalVXMLIncomingChannel : public PVXMLIncomingChannel
{
  PCLASSINFO(OpalVXMLIncomingChannel, PVXMLIncomingChannel)

  public:
    OpalVXMLIncomingChannel(OpalVXMLSession & vxml);

    PWAVFile * CreateWAVFile(const PFilePath & fn);
    virtual unsigned GetWavFileType() const = 0;
};

class OpalVXMLIncomingChannelPCM : public OpalVXMLIncomingChannel
{
  PCLASSINFO(OpalVXMLIncomingChannelPCM, OpalVXMLIncomingChannel)

  public:
    OpalVXMLIncomingChannelPCM(OpalVXMLSession & vxml);

    unsigned GetWavFileType() const
      { return PWAVFile::PCM_WavFile; }

  protected:
    // overrides from PVXMLIncomingChannel
    void DelayFrame(PINDEX len);
    BOOL WriteFrame(const void * buf, PINDEX len);
};

class OpalVXMLIncomingChannelG7231 : public OpalVXMLIncomingChannel
{
  PCLASSINFO(OpalVXMLIncomingChannelG7231, PVXMLIncomingChannel);

  public:
    OpalVXMLIncomingChannelG7231(OpalVXMLSession & vxml);

    unsigned GetWavFileType() const
      { return PWAVFile::fmt_MSG7231; }

  protected:
    // overrides from PVXMLIncomingChannel
    void DelayFrame(PINDEX len);
    BOOL WriteFrame(const void * buf, PINDEX len);
};

//////////////////////////////////////////////////////////////////

class OpalVXMLSession : public PVXMLSession 
{
  PCLASSINFO(OpalVXMLSession, PVXMLSession);
  public:
    OpalVXMLSession();

    // overrides from PVXMLSession
    BOOL PlayFile(const PString & fn);
    BOOL PlayData(const PBYTEArray & data);
    BOOL IsPlaying() const;

    BOOL StartRecording(const PFilePath & fn);
    BOOL EndRecording();
    BOOL IsRecording() const;

    // new functions
    virtual PVXMLOutgoingChannel * CreateOutgoingAudioChannel(H323AudioCodec & codec);
    virtual PVXMLIncomingChannel * CreateIncomingAudioChannel(H323AudioCodec & codec);

    virtual PVXMLIncomingChannel * SetIncomingAudioChannel(PVXMLIncomingChannel * chan);
    virtual PVXMLOutgoingChannel * SetOutgoingAudioChannel(PVXMLOutgoingChannel * chan);

  protected:
    PVXMLIncomingChannel * incomingChannel;
    PVXMLOutgoingChannel * outgoingChannel;
};

#endif

